
!UiTreeView methodsFor: 'initialization' stamp: 'mt 10/11/2010 17:23:49.302'!
initialize

	"Initialize headers."
	verticalHeader := Morph new color: Color transparent; width: self scrollBarThickness; changeTableLayout; listDirection: #topToBottom.
	horizontalHeader := Morph new color: Color transparent; height: self scrollBarThickness; changeTableLayout; listDirection: #leftToRight.
	vHeaderScroller := TransformMorph new color: Color transparent.
	hHeaderScroller := TransformMorph new color: Color transparent.

	super initialize.
	
	self
		borderWidth: 0;
		color: Color white.
	
	"Initialize viewport."
	self scroller addMorph: UiTreeViewport new.
	self connect: self viewport signal: #doubleClicked: toSelector: #toggleIndex:.
	self connect: self viewport signal: #decorationClicked: toSelector: #toggleIndex:.
	self connect: self viewport signal: #focusChanged: toSelector: #scrollToShow:.
	self viewport verticalHeader: verticalHeader.
	self viewport horizontalHeader: horizontalHeader.
	
	"Initialize item delegates."
	itemDelegates := Dictionary new.
	itemDelegates at: #default put: UiSimpleItemDelegate new.
	
	"Initialize headers - Finish."
	verticalHeader connect: self viewport signal: #resizedToWidth:height: toSelector: #height: pattern: #(2).
	vHeaderScroller addMorph: verticalHeader.

	horizontalHeader connect: self viewport signal: #resizedToWidth:height: toSelector: #width:.
	hHeaderScroller addMorph: horizontalHeader.

	self
		"addMorph: vHeaderScroller;"
		addMorph: hHeaderScroller.
	
	self resizeScroller.! !

!UiTreeView methodsFor: 'scrolling' stamp: 'mt 10/11/2010 17:35:05.273'!
scrollToShow: aRectangle
	"Scroll to include as much of aRectangle as possible, where aRectangle is in the scroller's local space."
	
	|offset|
	offset := self scroller offset.
	
	(aRectangle right - offset x) > self innerBounds width
		ifTrue: [offset := (aRectangle left - self innerBounds width + aRectangle width min: self viewport width - self innerBounds width) @ offset y].
	(aRectangle left - offset x) < 0
		ifTrue: [offset := (aRectangle left min: self viewport width - self innerBounds width) @ offset y].
	
	
	(aRectangle top - offset y) < 0
		ifTrue: [offset := offset x @ (
					(aRectangle top min: self viewport height - self innerBounds height))].
	(aRectangle bottom - offset y) > self innerBounds height
		ifTrue: [offset := offset x @ (
					(aRectangle top - self innerBounds height + aRectangle height min: self viewport height - self innerBounds height))].
		
	offset = self scroller offset ifFalse: [
		self scroller offset: offset.
		self setScrollDeltas]! !

!UiAbstractItemDelegate methodsFor: 'building' stamp: 'mt 10/11/2010 16:55:18.253'!
editorFor: aModelIndex
	"An editor morph must understand #accept."
	
	self subclassResponsibility.! !

!UiSimpleItemDelegate methodsFor: 'building' stamp: 'mt 10/11/2010 16:52:57.182'!
editorFor: aModelIndex

	^ UiLineEdit new
		theText: aModelIndex text.! !

!UiTreeViewport methodsFor: 'accessing' stamp: 'mt 10/11/2010 17:24:32.458'!
currentCell: aPoint

	aPoint = currentCell
		ifTrue: [^ self].
		
	currentCell := aPoint.
	self redrawLater.
	
	currentCell ifNotNil: [
		self focusChanged: (self cellBoundsFor: currentCell)].! !

!UiTreeViewport methodsFor: 'signals' stamp: 'mt 10/11/2010 17:18:00.533'!
focusChanged: aRectangle
	"Indicate that a scrollable container should ensure the given area as visible."
	
	self emit.! !

!UiTreeViewport methodsFor: 'event handling' stamp: 'mt 10/11/2010 16:53:44.92'!
handlesKeyboard: evt

	^ true! !

!UiTreeViewport methodsFor: 'accessing' stamp: 'mt 10/11/2010 17:13:05.489'!
hoveredCell: aPoint

	aPoint = hoveredCell
		ifTrue: [^ self].
		
	hoveredCell := aPoint.
	self redrawLater.! !

!UiTreeViewport methodsFor: 'event handling' stamp: 'mt 10/11/2010 17:17:04.271'!
keyStroke: evt

	evt keyCharacter = Character arrowUp
		ifTrue: [self currentCell: (self currentCell - (0@1) max: 1@1)].
		
	evt keyCharacter = Character arrowDown
		ifTrue: [self currentCell: (self currentCell + (0@1) min: (columns size @ rows size))].
		
	evt keyCharacter = Character arrowLeft
		ifTrue: [self currentCell: (self currentCell - (1@0) max: 1@1)].
		
	evt keyCharacter = Character arrowRight
		ifTrue: [self currentCell: (self currentCell + (1@0) min: (columns size @ rows size))].
		
	self selection
		add: self currentCell
		event: evt.! !

!UiTreeViewport methodsFor: 'event handling' stamp: 'mt 10/11/2010 17:05:42.001'!
mouseDown: evt

	self currentCell: (self cellIndexFromPoint: evt position).
	self selection
		add: self currentCell
		event: evt.
		
	evt hand
		waitForClicksOrDrag: self
		event: evt
		selectors: {#click:. #doubleClick:. nil. #startDrag:}
		threshold: 1. ! !

!UiTreeViewport methodsFor: 'event handling' stamp: 'mt 10/11/2010 16:57:04.191'!
mouseEnter: evt

	evt hand newKeyboardFocus: self.! !

!UiViewColumnInfo methodsFor: 'geometry' stamp: 'mt 10/11/2010 16:40:51.979'!
optimalSize
	"Return the width of the largest item."
	
	| max |
	max := 0.
	items do: [:item | 
		item hResizing ~= #spaceFill
			ifTrue: [max := max max: (item width + (item valueOfProperty: #indent))]].
	max := max max: self headerItem submorphBounds width + 10. "font rendering bugfix"
	^ max! !
